@@ -13,12 +13,13 @@ use serde::{
 
 use crate::{
     oid::ObjectId,
-    raw::{RawBinary, RAW_ARRAY_NEWTYPE, RAW_BSON_NEWTYPE, RAW_DOCUMENT_NEWTYPE},
+    raw::{RawBinaryRef, RAW_ARRAY_NEWTYPE, RAW_BSON_NEWTYPE, RAW_DOCUMENT_NEWTYPE},
     spec::{BinarySubtype, ElementType},
     uuid::UUID_NEWTYPE_NAME,
     Bson,
     DateTime,
     Decimal128,
+    DeserializerOptions,
     RawDocument,
     Timestamp,
 };
@@ -31,6 +32,7 @@ use super::{
     read_i64,
     read_string,
     read_u8,
+    DeserializerHint,
     Error,
     Result,
     MAX_BSON_SIZE,
@@ -38,22 +40,6 @@ use super::{
 };
 use crate::de::serde::MapDeserializer;
 
-/// Hint provided to the deserializer via `deserialize_newtype_struct` as to the type of thing
-/// being deserialized.
-#[derive(Debug, Clone, Copy)]
-enum DeserializerHint {
-    /// No hint provided, deserialize normally.
-    None,
-
-    /// The type being deserialized expects the BSON to contain a binary value with the provided
-    /// subtype. This is currently used to deserialize `bson::Uuid` values.
-    BinarySubtype(BinarySubtype),
-
-    /// The type being deserialized is raw BSON, meaning no allocations should occur as part of
-    /// deserializing and everything should be visited via borrowing or `Copy`.
-    RawBson,
-}
-
 /// Deserializer used to parse and deserialize raw BSON bytes.
 pub(crate) struct Deserializer<'de> {
     bytes: BsonBuf<'de>,
@@ -163,7 +149,7 @@ impl<'de> Deserializer<'de> {
                 let mut len = self.bytes.slice(4)?;
                 let len = read_i32(&mut len)?;
 
-                let doc = RawDocument::new(self.bytes.read_slice(len as usize)?)
+                let doc = RawDocument::from_bytes(self.bytes.read_slice(len as usize)?)
                     .map_err(Error::custom)?;
 
                 let access = if is_array {
@@ -185,7 +171,11 @@ impl<'de> Deserializer<'de> {
     where
         F: FnOnce(DocumentAccess<'_, 'de>) -> Result<O>,
     {
-        let mut length_remaining = read_i32(&mut self.bytes)? - 4;
+        let mut length_remaining = read_i32(&mut self.bytes)?;
+        if length_remaining < 4 {
+            return Err(Error::custom("invalid length, less than min document size"));
+        }
+        length_remaining -= 4;
         let out = f(DocumentAccess {
             root_deserializer: self,
             length_remaining: &mut length_remaining,
@@ -266,7 +256,7 @@ impl<'de> Deserializer<'de> {
                         visitor.visit_borrowed_bytes(self.bytes.read_slice(len as usize)?)
                     }
                     _ => {
-                        let binary = RawBinary::from_slice_with_len_and_payload(
+                        let binary = RawBinaryRef::from_slice_with_len_and_payload(
                             self.bytes.read_slice(len as usize)?,
                             len,
                             subtype,
@@ -307,8 +297,11 @@ impl<'de> Deserializer<'de> {
                     )),
                     _ => {
                         let code = read_string(&mut self.bytes, utf8_lossy)?;
-                        let doc = Bson::JavaScriptCode(code).into_extended_document();
-                        visitor.visit_map(MapDeserializer::new(doc))
+                        let doc = Bson::JavaScriptCode(code).into_extended_document(false);
+                        visitor.visit_map(MapDeserializer::new(
+                            doc,
+                            DeserializerOptions::builder().human_readable(false).build(),
+                        ))
                     }
                 }
             }
@@ -361,8 +354,11 @@ impl<'de> Deserializer<'de> {
                     )),
                     _ => {
                         let symbol = read_string(&mut self.bytes, utf8_lossy)?;
-                        let doc = Bson::Symbol(symbol).into_extended_document();
-                        visitor.visit_map(MapDeserializer::new(doc))
+                        let doc = Bson::Symbol(symbol).into_extended_document(false);
+                        visitor.visit_map(MapDeserializer::new(
+                            doc,
+                            DeserializerOptions::builder().human_readable(false).build(),
+                        ))
                     }
                 }
             }
@@ -648,13 +644,36 @@ impl<'d, 'de> serde::de::Deserializer<'de> for DocumentKeyDeserializer<'d, 'de>
         }
     }
 
+    fn deserialize_enum<V>(
+        self,
+        _name: &str,
+        _variants: &'static [&'static str],
+        visitor: V,
+    ) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_enum(
+            self.root_deserializer
+                .deserialize_cstr()?
+                .into_deserializer(),
+        )
+    }
+
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_newtype_struct(self)
+    }
+
     fn is_human_readable(&self) -> bool {
         false
     }
 
     forward_to_deserialize_any! {
         bool char str bytes byte_buf option unit unit_struct string
-        identifier newtype_struct seq tuple tuple_struct struct map enum
+        identifier seq tuple tuple_struct struct map
         ignored_any i8 i16 i32 i64 u8 u16 u32 u64 f32 f64
     }
 }
@@ -674,13 +693,20 @@ impl<'de> serde::de::Deserializer<'de> for FieldDeserializer {
         visitor.visit_borrowed_str(self.field_name)
     }
 
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_newtype_struct(self)
+    }
+
     fn is_human_readable(&self) -> bool {
         false
     }
 
     serde::forward_to_deserialize_any! {
         bool u8 u16 u32 u64 i8 i16 i32 i64 f32 f64 char str string seq
-        bytes byte_buf map struct option unit newtype_struct
+        bytes byte_buf map struct option unit
         ignored_any unit_struct tuple_struct tuple enum identifier
     }
 }
@@ -974,9 +1000,7 @@ impl<'de, 'a> serde::de::Deserializer<'de> for &'a mut TimestampDeserializer {
         match self.stage {
             TimestampDeserializationStage::TopLevel => {
                 self.stage = TimestampDeserializationStage::Time;
-                visitor.visit_map(TimestampAccess {
-                    deserializer: &mut self,
-                })
+                visitor.visit_map(TimestampAccess { deserializer: self })
             }
             TimestampDeserializationStage::Time => {
                 self.stage = TimestampDeserializationStage::Increment;
@@ -1081,9 +1105,7 @@ impl<'de, 'a> serde::de::Deserializer<'de> for &'a mut DateTimeDeserializer {
                 }
                 _ => {
                     self.stage = DateTimeDeserializationStage::NumberLong;
-                    visitor.visit_map(DateTimeAccess {
-                        deserializer: &mut self,
-                    })
+                    visitor.visit_map(DateTimeAccess { deserializer: self })
                 }
             },
             DateTimeDeserializationStage::NumberLong => {
@@ -1096,13 +1118,20 @@ impl<'de, 'a> serde::de::Deserializer<'de> for &'a mut DateTimeDeserializer {
         }
     }
 
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_newtype_struct(self)
+    }
+
     fn is_human_readable(&self) -> bool {
         false
     }
 
     serde::forward_to_deserialize_any! {
         bool u8 u16 u32 u64 i8 i16 i32 i64 f32 f64 char str string seq
-        bytes byte_buf map struct option unit newtype_struct
+        bytes byte_buf map struct option unit
         ignored_any unit_struct tuple_struct tuple enum identifier
     }
 }
@@ -1147,13 +1176,13 @@ impl<'de, 'd> serde::de::MapAccess<'de> for BinaryAccess<'d, 'de> {
 }
 
 struct BinaryDeserializer<'a> {
-    binary: RawBinary<'a>,
+    binary: RawBinaryRef<'a>,
     hint: DeserializerHint,
     stage: BinaryDeserializationStage,
 }
 
 impl<'a> BinaryDeserializer<'a> {
-    fn new(binary: RawBinary<'a>, hint: DeserializerHint) -> Self {
+    fn new(binary: RawBinaryRef<'a>, hint: DeserializerHint) -> Self {
         Self {
             binary,
             hint,
@@ -1172,9 +1201,7 @@ impl<'de, 'a> serde::de::Deserializer<'de> for &'a mut BinaryDeserializer<'de> {
         match self.stage {
             BinaryDeserializationStage::TopLevel => {
                 self.stage = BinaryDeserializationStage::Subtype;
-                visitor.visit_map(BinaryAccess {
-                    deserializer: &mut self,
-                })
+                visitor.visit_map(BinaryAccess { deserializer: self })
             }
             BinaryDeserializationStage::Subtype => {
                 self.stage = BinaryDeserializationStage::Bytes;
@@ -1329,13 +1356,20 @@ impl<'de, 'a, 'b> serde::de::Deserializer<'de> for &'b mut CodeWithScopeDeserial
         }
     }
 
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_newtype_struct(self)
+    }
+
     fn is_human_readable(&self) -> bool {
         false
     }
 
     serde::forward_to_deserialize_any! {
         bool u8 u16 u32 u64 i8 i16 i32 i64 f32 f64 char str string seq
-        bytes byte_buf map struct option unit newtype_struct
+        bytes byte_buf map struct option unit
         ignored_any unit_struct tuple_struct tuple enum identifier
     }
 }
@@ -1438,13 +1472,20 @@ impl<'de, 'a, 'b> serde::de::Deserializer<'de> for &'b mut DbPointerDeserializer
         }
     }
 
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_newtype_struct(self)
+    }
+
     fn is_human_readable(&self) -> bool {
         false
     }
 
     serde::forward_to_deserialize_any! {
         bool u8 u16 u32 u64 i8 i16 i32 i64 f32 f64 char str string seq
-        bytes byte_buf map struct option unit newtype_struct
+        bytes byte_buf map struct option unit
         ignored_any unit_struct tuple_struct tuple enum identifier
     }
 }
@@ -1646,13 +1687,20 @@ impl<'de, 'a> serde::de::Deserializer<'de> for RawBsonDeserializer<'de> {
         }
     }
 
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
+    where
+        V: serde::de::Visitor<'de>,
+    {
+        visitor.visit_newtype_struct(self)
+    }
+
     fn is_human_readable(&self) -> bool {
         false
     }
 
     serde::forward_to_deserialize_any! {
         bool u8 u16 u32 u64 i8 i16 i32 i64 f32 f64 char str string seq
-        bytes byte_buf map struct option unit newtype_struct
+        bytes byte_buf map struct option unit
         ignored_any unit_struct tuple_struct tuple enum identifier
     }
 }
