@@ -51,7 +51,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! bson = "2.1.0"
+//! bson = "2.2.0"
 //! ```
 //!
 //! Note that if you are using `bson` through the `mongodb` crate, you do not need to specify it in
@@ -264,7 +264,7 @@
 //! it will only happen in a minor or major version release.
 
 #![allow(clippy::cognitive_complexity)]
-#![doc(html_root_url = "https://docs.rs/bson/2.1.0")]
+#![doc(html_root_url = "https://docs.rs/bson/2.2.0")]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
 #[doc(inline)]
@@ -277,15 +277,17 @@ pub use self::{
         DeserializerOptions,
     },
     decimal128::Decimal128,
+    raw::{
+        RawArray, RawArrayBuf, RawBinaryRef, RawBson, RawBsonRef, RawDbPointerRef, RawDocument,
+        RawDocumentBuf, RawJavaScriptCodeWithScope, RawJavaScriptCodeWithScopeRef, RawRegexRef,
+    },
     ser::{
-        to_bson, to_bson_with_options, to_document, to_document_with_options, to_vec, Serializer,
-        SerializerOptions,
+        to_bson, to_bson_with_options, to_document, to_document_with_options, to_raw_document_buf,
+        to_vec, Serializer, SerializerOptions,
     },
     uuid::{Uuid, UuidRepresentation},
 };
 
-pub(crate) use self::raw::RawDocument;
-
 #[macro_use]
 mod macros;
 mod bson;
@@ -295,7 +297,7 @@ pub mod decimal128;
 pub mod document;
 pub mod extjson;
 pub mod oid;
-pub(crate) mod raw;
+pub mod raw;
 pub mod ser;
 pub mod serde_helpers;
 pub mod spec;
