@@ -252,7 +252,7 @@ pub mod rfc3339_string_as_bson_datetime {
         D: Deserializer<'de>,
     {
         let date = DateTime::deserialize(deserializer)?;
-        Ok(date.to_rfc3339())
+        Ok(date.to_rfc3339_string())
     }
 
     /// Serializes an ISO string as a DateTime.
@@ -297,7 +297,7 @@ pub mod bson_datetime_as_rfc3339_string {
 
     /// Serializes a [`crate::DateTime`] as an RFC 3339 (ISO 8601) formatted string.
     pub fn serialize<S: Serializer>(val: &DateTime, serializer: S) -> Result<S::Ok, S::Error> {
-        serializer.serialize_str(&val.to_rfc3339())
+        serializer.serialize_str(&val.to_rfc3339_string())
     }
 }
 
@@ -355,16 +355,14 @@ pub mod hex_string_as_object_id {
 #[cfg(feature = "uuid-0_8")]
 #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
 pub mod uuid_as_binary {
-    use crate::{spec::BinarySubtype, Binary};
-    use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
+    use serde::{Deserialize, Deserializer, Serialize, Serializer};
     use std::result::Result;
     use uuid::Uuid;
 
     /// Serializes a Uuid as a Binary.
     #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
     pub fn serialize<S: Serializer>(val: &Uuid, serializer: S) -> Result<S::Ok, S::Error> {
-        let binary: Binary = (*val).into();
-        binary.serialize(serializer)
+        crate::uuid::Uuid::from_external_uuid(*val).serialize(serializer)
     }
 
     /// Deserializes a Uuid from a Binary.
@@ -373,22 +371,8 @@ pub mod uuid_as_binary {
     where
         D: Deserializer<'de>,
     {
-        let binary = Binary::deserialize(deserializer)?;
-        if binary.subtype == BinarySubtype::Uuid {
-            if binary.bytes.len() == 16 {
-                let mut bytes = [0u8; 16];
-                bytes.copy_from_slice(&binary.bytes);
-                Ok(Uuid::from_bytes(bytes))
-            } else {
-                Err(de::Error::custom(
-                    "cannot convert Binary to Uuid: incorrect bytes length",
-                ))
-            }
-        } else {
-            Err(de::Error::custom(
-                "cannot convert Binary to Uuid: incorrect binary subtype",
-            ))
-        }
+        let bson_uuid = crate::uuid::Uuid::deserialize(deserializer)?;
+        Ok(bson_uuid.into())
     }
 }
 
@@ -412,7 +396,7 @@ pub mod uuid_as_binary {
 #[cfg(feature = "uuid-0_8")]
 #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
 pub mod uuid_as_java_legacy_binary {
-    use crate::{spec::BinarySubtype, Binary};
+    use crate::{uuid::UuidRepresentation, Binary};
     use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
     use std::result::Result;
     use uuid::Uuid;
@@ -420,13 +404,10 @@ pub mod uuid_as_java_legacy_binary {
     /// Serializes a Uuid as a Binary in a Java Legacy UUID format.
     #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
     pub fn serialize<S: Serializer>(val: &Uuid, serializer: S) -> Result<S::Ok, S::Error> {
-        let mut bytes = val.as_bytes().to_vec();
-        bytes[0..8].reverse();
-        bytes[8..16].reverse();
-        let binary = Binary {
-            subtype: BinarySubtype::UuidOld,
-            bytes,
-        };
+        let binary = Binary::from_uuid_with_representation(
+            crate::uuid::Uuid::from_external_uuid(*val),
+            UuidRepresentation::JavaLegacy,
+        );
         binary.serialize(serializer)
     }
 
@@ -437,17 +418,10 @@ pub mod uuid_as_java_legacy_binary {
         D: Deserializer<'de>,
     {
         let binary = Binary::deserialize(deserializer)?;
-        if binary.subtype != BinarySubtype::UuidOld {
-            Err(de::Error::custom("expecting BinarySubtype::UuidOld"))
-        } else if binary.bytes.len() != 16 {
-            Err(de::Error::custom("expecting 16 bytes"))
-        } else {
-            let mut buf = [0u8; 16];
-            buf.copy_from_slice(&binary.bytes);
-            buf[0..8].reverse();
-            buf[8..16].reverse();
-            Ok(Uuid::from_bytes(buf))
-        }
+        let uuid = binary
+            .to_uuid_with_representation(UuidRepresentation::JavaLegacy)
+            .map_err(de::Error::custom)?;
+        Ok(uuid.to_uuid_0_8())
     }
 }
 
@@ -471,7 +445,7 @@ pub mod uuid_as_java_legacy_binary {
 #[cfg(feature = "uuid-0_8")]
 #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
 pub mod uuid_as_python_legacy_binary {
-    use crate::{spec::BinarySubtype, Binary};
+    use crate::{uuid::UuidRepresentation, Binary};
     use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
     use std::result::Result;
     use uuid::Uuid;
@@ -479,10 +453,10 @@ pub mod uuid_as_python_legacy_binary {
     /// Serializes a Uuid as a Binary in a Python Legacy UUID format.
     #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
     pub fn serialize<S: Serializer>(val: &Uuid, serializer: S) -> Result<S::Ok, S::Error> {
-        let binary = Binary {
-            subtype: BinarySubtype::UuidOld,
-            bytes: val.as_bytes().to_vec(),
-        };
+        let binary = Binary::from_uuid_with_representation(
+            crate::uuid::Uuid::from_external_uuid(*val),
+            UuidRepresentation::PythonLegacy,
+        );
         binary.serialize(serializer)
     }
 
@@ -493,15 +467,10 @@ pub mod uuid_as_python_legacy_binary {
         D: Deserializer<'de>,
     {
         let binary = Binary::deserialize(deserializer)?;
-        if binary.subtype != BinarySubtype::UuidOld {
-            Err(de::Error::custom("expecting BinarySubtype::UuidOld"))
-        } else if binary.bytes.len() != 16 {
-            Err(de::Error::custom("expecting 16 bytes"))
-        } else {
-            let mut buf = [0u8; 16];
-            buf.copy_from_slice(&binary.bytes);
-            Ok(Uuid::from_bytes(buf))
-        }
+        let uuid = binary
+            .to_uuid_with_representation(UuidRepresentation::PythonLegacy)
+            .map_err(de::Error::custom)?;
+        Ok(uuid.into())
     }
 }
 
@@ -525,7 +494,7 @@ pub mod uuid_as_python_legacy_binary {
 #[cfg(feature = "uuid-0_8")]
 #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
 pub mod uuid_as_c_sharp_legacy_binary {
-    use crate::{spec::BinarySubtype, Binary};
+    use crate::{uuid::UuidRepresentation, Binary};
     use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
     use std::result::Result;
     use uuid::Uuid;
@@ -533,14 +502,10 @@ pub mod uuid_as_c_sharp_legacy_binary {
     /// Serializes a Uuid as a Binary in a C# Legacy UUID format.
     #[cfg_attr(docsrs, doc(cfg(feature = "uuid-0_8")))]
     pub fn serialize<S: Serializer>(val: &Uuid, serializer: S) -> Result<S::Ok, S::Error> {
-        let mut bytes = val.as_bytes().to_vec();
-        bytes[0..4].reverse();
-        bytes[4..6].reverse();
-        bytes[6..8].reverse();
-        let binary = Binary {
-            subtype: BinarySubtype::UuidOld,
-            bytes,
-        };
+        let binary = Binary::from_uuid_with_representation(
+            crate::uuid::Uuid::from_external_uuid(*val),
+            UuidRepresentation::CSharpLegacy,
+        );
         binary.serialize(serializer)
     }
 
@@ -551,18 +516,10 @@ pub mod uuid_as_c_sharp_legacy_binary {
         D: Deserializer<'de>,
     {
         let binary = Binary::deserialize(deserializer)?;
-        if binary.subtype != BinarySubtype::UuidOld {
-            Err(de::Error::custom("expecting BinarySubtype::UuidOld"))
-        } else if binary.bytes.len() != 16 {
-            Err(de::Error::custom("expecting 16 bytes"))
-        } else {
-            let mut buf = [0u8; 16];
-            buf.copy_from_slice(&binary.bytes);
-            buf[0..4].reverse();
-            buf[4..6].reverse();
-            buf[6..8].reverse();
-            Ok(Uuid::from_bytes(buf))
-        }
+        let uuid = binary
+            .to_uuid_with_representation(UuidRepresentation::CSharpLegacy)
+            .map_err(de::Error::custom)?;
+        Ok(uuid.into())
     }
 }
 
